#!/usr/bin/env python
# -*- coding: utf-8 -*-
# pylint: disable=invalid-name, too-many-arguments, broad-except,
# pylint: disable=missing-docstring, line-too-long
# pylint: anomalous-backslash-in-string

import time
import requests
import argparse
import multiprocessing
from prettytable import PrettyTable

from utils import ColorString
from timePatch import get_latency_str_for_millisecond

#######################################################
# GLOBAL variable
RES_QUEUE = []
ERROR_C = 0
proc_current_pos = 0
proc_current_persent = 0


#######################################################
def post_unit(session, requests_url):
    if isinstance(requests_url, str) is False:
        [session, requests_url] = [requests_url, session]

    try:
        start_t = time.time()
        res = session.get(requests_url, verify=True)
        proc_t = time.time() - start_t
        return [res.status_code, proc_t]
    except Exception as error:
        print error
        return [0, 0]


class Rebot(object):

    def __init__(self, requests_url, thread_c, requests_c):
        """
        :param requests_c:
        :return:
        """
        self.session = requests.session()
        self.requests_url = requests_url
        self.requests_c = requests_c
        self.thread_c = thread_c
        self.multi_pool = multiprocessing.Pool(processes=self.thread_c)

    def thread_callback(self, res):
        global ERROR_C
        global proc_current_pos
        global proc_current_persent

        proc_current_pos += 1
        persent = proc_current_pos * 100 / self.requests_c
        # if persent > proc_current_persent:
        #     proc_current_persent = persent
        #     print fmt(color.YELLOW, 'proc...%s%%' % proc_current_persent)
        #
        # if res[0] != 200:
        #     ERROR_C += 1
        # RES_QUEUE.append(res[1])

    def run(self):
        for i in range(self.requests_c):
            self.multi_pool.apply_async(post_unit, args={self.session, self.requests_url}, callback=self.thread_callback)
        self.multi_pool.close()
        self.multi_pool.join()


def results():
    print fmt(color.CYAN, '*' * 50)
    print fmt(color.RED, "请求失败：%s (%d%%)" % (ERROR_C, (ERROR_C * 100 / proc_current_pos)))
    print fmt(color.GREEN, "总请求量：%s" % proc_current_pos)
    print fmt(color.GREEN, "耗时：%0.2fs" % proc_t)
    print fmt(color.CYAN, '*' * 50)

    res_dic = dict()
    for i in RES_QUEUE:
        latency = get_latency_str_for_millisecond(i * 1000)
        if res_dic.get(latency, None) is None:
            res_dic[latency] = 0
        res_dic[latency] += 1

    x = PrettyTable(["datetime", "count"])
    x.padding_width = 1
    for i in res_dic.keys():
        x.add_row([i, res_dic[i]])
    print x


#######################################################
def init_config():
    """
    Init Script & get config variable
    :return:[List][requests_url, requests_c, thread_c]
    """
    options = define_options()
    args = options.parse_args()
    return [args.url, int(args.p), int(args.n)]


def main():
    print ColorString.out('[* start *]', ColorString.GREEN)

    # Read Config
    [requests_url, thread_c, requests_c] = init_config()
    # proc_start_t = time.time()
    # proc_t = time.time() - proc_start_t

    rebot = Rebot(requests_url, thread_c, requests_c)
    rebot.run()

    print ColorString.out('[* end *]', ColorString.GREEN)


def define_options():
    """
    :return:
    """
    parser = argparse.ArgumentParser()
    parser.add_argument('-n', required=False, default=10, help="")
    parser.add_argument('-p', required=False, default=multiprocessing.cpu_count(), help="")
    parser.add_argument('--url', required=True, help="")
    return parser


# ------------------------
#   Demo
# ------------------------
if __name__ == '__main__':
    main()
